
#include <WiFi.h>
#include <AsyncTCP.h>          // https://github.com/me-no-dev/AsyncTCP/archive/master.zip
#include <ESPAsyncWebServer.h> // https://github.com/me-no-dev/ESPAsyncWebServer/archive/master.zip

#define SSID "xxxxxxxxxx"
#define PSW  "xxxxxxxxxx"

// Servo
#include <Servo.h>
#define SERVO_PIN 19

class Aig : public Servo
{
  public:
    uint8_t  servoPin;
    uint16_t posDroit;
    uint16_t posDev;
    uint16_t curPos;
    uint16_t speed;
    bool     sens;
    bool     test;
    uint32_t nextProcess;

    Aig(uint8_t); // Constructor
    void loop();
    void setup();
    void fnTest();
};

// Constructor
Aig::Aig(uint8_t pin) :
  servoPin(pin),
  posDroit(1500),
  posDev(1500),
  curPos(1500),
  speed(50),
  sens(0),
  test(false),
  nextProcess(0)
{}

void Aig::setup()
{
  Aig::attach(servoPin);
  Aig::writeMicroseconds(curPos);
}

void Aig::fnTest()
{
  if (millis() > nextProcess) {
    if (test) {
      if ((posDroit < posDev) && (curPos == posDev))
        sens = 0;
      if ((posDroit < posDev) && (curPos == posDroit))
        sens = 1;
      if ((posDev < posDroit) && (curPos == posDev))
        sens = 1;
      if ((posDev < posDroit) && (curPos == posDroit))
        sens = 0;

      if (sens == 0)
        curPos--;
      else
        curPos++;

      Serial.println(curPos);
      Aig::writeMicroseconds(curPos);
      if (curPos == posDev || curPos == posDroit) {
        test = false;
        Serial.println("end");
      }
    }
    nextProcess += speed;
  }
}

Aig aig(SERVO_PIN); // Instance pour aiguille

AsyncWebServer server(80);
AsyncWebSocket ws("/ws");


const char index_html[] PROGMEM = R"rawliteral(

<!DOCTYPE HTML>
<html>
<head>
<title>ESP Web Server</title>
<meta name="viewport" content="width=device-width, initial-scale=1">
<link rel="icon" href="data:,">
<style>
html {
  font-family: Arial, Helvetica, sans-serif;
  text-align: center;
}
h2 {
  font-size: 1.5rem;
  font-weight: bold;
  color: #143642;
}
body {
  margin: 0;
}
.content {
  padding: 30px;
  max-width: 600px;
  margin: 0 auto;
}
.card {
  background-color: #F8F7F9;
  ;
  box-shadow: 2px 2px 12px 1px rgba(140,140,140,.5);
  padding-top: 10px;
  padding-bottom: 20px;
}
.button {
  padding: 10px 40px;
  font-size: 18px;
  text-align: center;
  outline: none;
  color: #666;
  background-color: #9F9;
  border: none;
  border-radius: 5px;
  -webkit-touch-callout: none;
  -webkit-user-select: none;
  -khtml-user-select: none;
  -moz-user-select: none;
  -ms-user-select: none;
  user-select: none;
  -webkit-tap-highlight-color: rgba(0,0,0,0);
}
/*.button:hover {background-color: #0f8b8d}*/
.button:active {
  background-color: #0f8b8d;
  box-shadow: 2 2px #CDCDCD;
  transform: translateY(2px);
}
.state {
  font-size: 1.2rem;
  color: #8c8c8c;
  font-weight: bold;
}
#tab2 {
  margin-left: auto;
    margin-right: auto;
}
</style>
</head>

<body>

<p class="state">Connexion : <span id="messages"></span></p>
<div class="content">
  <div class="card">
    <h2>Servo aiguillage</h2>
    <!--<p class="state">state: <span id="state">%STATE%</span></p>-->
    <table width="90%" id="tab2">
      <tr class="tr_01">
        <td>Vitesse</td>
        <td><input id="aigSpeedSlider" type="range" min="0.0" max="1.0" step="0.01" value="0.5" onInput="aigSpeedSlider(this.value)"/></td>
        <td><input id="aigSpeedInput"type="number" min="0.0" max="1.0" step="0.01" value="0.5"></td>
      </tr>
      <tr class="tr_01">
        <td>Position droite</td>
        <td><input id="aigPosDroitSlider" type="range" min="800" max="2500" step="1" value="1500" onInput="aigPosSlider('aigPosDroitInput', this.value)"/></td>
        <td><input type="number" min="800" max="2500" id="aigPosDroitInput" value="1500"></td>
      </tr>
      <tr class="tr_01">
        <td>Position d&eacute;vi&eacute;e</td>
        <td><input id="aigPosDevieSlider" type="range" min="800" max="2500" step="1" value="1500" onInput=
"aigPosSlider('aigPosDevieInput', this.value)"/></td>
        <td><input type="number" min="800" max="2500" id="aigPosDevieInput" value="1500"></td>
      </tr>
    </table>
    <p><button id="test" class="button">Tester</button></p>
  </div>
</div>
<script>
  var gateway = `ws://${window.location.hostname}/ws`;
  var websocket;
  window.addEventListener('load', onLoad);
  function initWebSocket() {
    console.log('Trying to open a WebSocket connection...');
    websocket = new WebSocket(gateway);
    websocket.onopen    = onOpen;
    websocket.onclose   = onClose;
    websocket.onmessage = onMessage; // <-- add this line
  }
  function onOpen(event) {
    console.log('Connection opened');
    document.getElementById('messages').innerHTML = "Connected";
  }
  
  function onClose(event) {
    console.log('Connection closed');
    setTimeout(initWebSocket, 2000);
  }
  
  function onMessage(event) {
    //document.getElementById('state').innerHTML = event.data;
    console.debug("WebSocket message received:", event);
    console.log(`Received a notification from ${event.origin}`);
    console.log("Received event.data : " + event.data);
  }
  
  function onLoad(event) {
    initWebSocket();
    initButton();
  }
  function initButton() {
    document.getElementById('test').addEventListener('click', testFn);
  }
  
  function testFn() {
    var myArray = new ArrayBuffer(8);
    var longInt8View = new Uint8Array(myArray);
    longInt8View[0] = 0x10; // ID satellite
    longInt8View[1] = 0xF3; // Fonction test
    websocket.send(myArray);
  }
  
  var aigSpeedSlider = function (speed) { 
    document.getElementById("aigSpeedInput").value = speed;
    var myArray = new ArrayBuffer(8);
    var longInt8View = new Uint8Array(myArray);
    longInt8View[0] = 0x10; // ID satellite
    longInt8View[1] = 0xF2; // Reglage vitesse
    longInt8View[2] = speed * 100;
    websocket.send(myArray);
  }
  
  var aigPosSlider = function (id, pos) { 
    document.getElementById(id).value = pos;
    var myArray = new ArrayBuffer(8);
    var longInt8View = new Uint8Array(myArray);
    longInt8View[0] = 0x10; // ID satellite
    longInt8View[1] = 0xF1; // Reglage positions
    if(id == "aigPosDroitInput")
      longInt8View[2] = 0;
    else
      longInt8View[2] = 1;
    longInt8View[3] = pos >> 8;
    longInt8View[4] = pos & 0x00FF;
    websocket.send(myArray);
  }
</script>
</body>
</html>

)rawliteral";



void onEvent(AsyncWebSocket *server, AsyncWebSocketClient *client, AwsEventType type,
             void *arg, uint8_t *data, size_t len) {
  switch (type) {
    case WS_EVT_CONNECT:
      Serial.printf("WebSocket client #%u connected from %s\n", client->id(), client->remoteIP().toString().c_str());
      break;
    case WS_EVT_DISCONNECT:
      Serial.printf("WebSocket client #%u disconnected\n", client->id());
      break;
    case WS_EVT_DATA:
      AwsFrameInfo * info = (AwsFrameInfo*)arg;
      if(info->final && info->index == 0 && info->len == len)
       reglage(data);
  }
}

void initWebSocket() {
  ws.onEvent(onEvent);
  server.addHandler(&ws);
}

String processor(const String& var)
{
  Serial.println(var);
  if(var == "STATE")
      return "ON";
  return String();
}

void reglage(uint8_t* data)
{
  uint16_t valueServo;
  if(data[1] == 0xF1)
    valueServo = data[3] << 8 | data[4];
  switch(data[1]) {
    case 0xF1 : // Reglage de position
      if(data[2] == 0) {
        aig.posDroit = valueServo;
        Serial.printf("Position aiguille droite : ");
      }
      else {
        aig.posDev = valueServo;
        Serial.printf("Position aiguille déviée : ");
      }
      Serial.printf("%d\n",valueServo);
      aig.curPos = valueServo;
      aig.writeMicroseconds(valueServo);
      break;
    case 0xF2 : // Reglage de vitesse
    Serial.print(data[2]);
      aig.speed = 100 - data[2];
      break;
    case 0xF3 : // Test
      aig.test = true;
      Serial.print("Test : ");
      Serial.println(aig.test ? "oui" : "non");
      break; 
  }
}

void setup()
{
  Serial.begin(115200);
  //Serial2.begin(115200, SERIAL_8N1, RXD2, TXD2);
  
  // Connect to Wi-Fi
  WiFi.begin(SSID, PSW);
  while (WiFi.status() != WL_CONNECTED) {
    delay(1000);
    Serial.println("Connecting to WiFi..");
  }
  // Print ESP Local IP Address
  Serial.println(WiFi.localIP());
  initWebSocket();

  // Route for root / web page
  server.on("/", HTTP_GET, [](AsyncWebServerRequest *request){
    request->send_P(200, "text/html", index_html, processor);
  });

  server.begin();
  aig.setup();
}


void loop()
{
//  ws.cleanupClients(); 
//  aig.fnTest();
}
